@@ -51,6 +51,7 @@ static int calc_header_length( u32 len );
 static int write_16(IOBUF inp, u16 a);
 static int write_32(IOBUF inp, u32 a);
 static int write_header( IOBUF out, int ctb, u32 len );
+static int write_sign_packet_header( IOBUF out, int ctb, u32 len );
 static int write_header2( IOBUF out, int ctb, u32 len, int hdrlen, int blkmode );
 static int write_new_header( IOBUF out, int ctb, u32 len, int hdrlen );
 static int write_version( IOBUF out, int ctb );
@@ -300,18 +301,26 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
 	    iobuf_put(a, sk->protect.s2k.mode );
 	    iobuf_put(a, sk->protect.s2k.hash_algo );
 	    if( sk->protect.s2k.mode == 1
-		|| sk->protect.s2k.mode == 4 )
+		|| sk->protect.s2k.mode == 3 )
 		iobuf_write(a, sk->protect.s2k.salt, 8 );
-	    if( sk->protect.s2k.mode == 4 )
-		write_32(a, sk->protect.s2k.count );
+	    if( sk->protect.s2k.mode == 3 )
+		iobuf_put(a, sk->protect.s2k.count );
 	    iobuf_write(a, sk->protect.iv, 8 );
 	}
     }
     else
 	iobuf_put(a, 0 );
-    for(   ; i < nskey; i++ )
-	mpi_write(a, sk->skey[i] );
-    write_16(a, sk->csum );
+    if( sk->is_protected && sk->version >= 4 ) {
+	byte *p;
+	assert( mpi_is_opaque( sk->skey[npkey] ) );
+	p = mpi_get_opaque( sk->skey[npkey], &i );
+	iobuf_write(a, p, i );
+    }
+    else {
+	for(   ; i < nskey; i++ )
+	    mpi_write(a, sk->skey[i] );
+	write_16(a, sk->csum );
+    }
 
   leave:
     write_header2(out, ctb, iobuf_get_temp_length(a), sk->hdrbytes, 1 );
@@ -330,17 +339,17 @@ do_symkey_enc( IOBUF out, int ctb, PKT_symkey_enc *enc )
 
     assert( enc->version == 4 );
     switch( enc->s2k.mode ) {
-      case 0: case 1: case 4: break;
+      case 0: case 1: case 3: break;
       default: log_bug("do_symkey_enc: s2k=%d\n", enc->s2k.mode );
     }
     iobuf_put( a, enc->version );
     iobuf_put( a, enc->cipher_algo );
     iobuf_put( a, enc->s2k.mode );
     iobuf_put( a, enc->s2k.hash_algo );
-    if( enc->s2k.mode == 1 || enc->s2k.mode == 4 ) {
+    if( enc->s2k.mode == 1 || enc->s2k.mode == 3 ) {
 	iobuf_write(a, enc->s2k.salt, 8 );
-	if( enc->s2k.mode == 4 )
-	    write_32(a, enc->s2k.count);
+	if( enc->s2k.mode == 3 )
+	    iobuf_put(a, enc->s2k.count);
     }
     if( enc->seskeylen )
 	iobuf_write(a, enc->seskey, enc->seskeylen );
@@ -669,7 +678,10 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
     for(i=0; i < n; i++ )
 	mpi_write(a, sig->data[i] );
 
-    write_header(out, ctb, iobuf_get_temp_length(a) );
+    if( is_RSA(sig->pubkey_algo) && sig->version < 4 )
+	write_sign_packet_header(out, ctb, iobuf_get_temp_length(a) );
+    else
+	write_header(out, ctb, iobuf_get_temp_length(a) );
     if( iobuf_write_temp( out, a ) )
 	rc = G10ERR_WRITE_FILE;
 
@@ -747,6 +759,18 @@ write_header( IOBUF out, int ctb, u32 len )
     return write_header2( out, ctb, len, 0, 1 );
 }
 
+
+static int
+write_sign_packet_header( IOBUF out, int ctb, u32 len )
+{
+    /* work around a bug in the pgp read function for signature packets,
+     * which are not correctly coded and silently assume at some
+     * point 2 byte length headers.*/
+    iobuf_put(out, 0x89 );
+    iobuf_put(out, len >> 8 );
+    return iobuf_put(out, len ) == -1 ? -1:0;
+}
+
 /****************
  * if HDRLEN is > 0, try to build a header of this length.
  * we need this, so that we can hash packets without reading them again.
