@@ -25,6 +25,7 @@
 #include <string.h>
 #include <unistd.h>
 
+/* #define MAINTAINER_OPTIONS */
 
 #include "packet.h"
 #include "iobuf.h"
@@ -51,42 +52,43 @@ static ARGPARSE_OPTS opts[] = {
     { 300, NULL, 0, N_("@Commands:\n ") },
 
   #ifdef IS_G10
-    { 's', "sign",      0, N_("|[file]|make a signature")},
-    { 539, "clearsign", 0, N_("|[file]|make a clear text signature") },
-    { 'b', "detach-sign", 0, N_("make a detached signature")},
-    { 'e', "encrypt",   0, N_("encrypt data")},
-    { 'c', "symmetric", 0, N_("encryption only with symmetric cipher")},
-    { 507, "store",     0, N_("store only")},
-    { 'd', "decrypt",   0, N_("decrypt data (default)")},
-    { 550, "verify"   , 0, N_("verify a signature")},
+    { 's', "sign",      256, N_("|[file]|make a signature")},
+    { 539, "clearsign", 256, N_("|[file]|make a clear text signature") },
+    { 'b', "detach-sign", 256, N_("make a detached signature")},
+    { 'e', "encrypt",   256, N_("encrypt data")},
+    { 'c', "symmetric", 256, N_("encryption only with symmetric cipher")},
+    { 507, "store",     256, N_("store only")},
+    { 'd', "decrypt",   256, N_("decrypt data (default)")},
+    { 550, "verify"   , 256, N_("verify a signature")},
   #endif
-    { 551, "list-keys", 0, N_("list keys")},
-    { 552, "list-sigs", 0, N_("list keys and signatures")},
-    { 508, "check-sigs",0, N_("check key signatures")},
-    { 515, "fingerprint", 0, N_("list keys and fingerprints")},
-    { 558, "list-secret-keys", 0, N_("list secret keys")},
+    { 551, "list-keys", 256, N_("list keys")},
+    { 552, "list-sigs", 256, N_("list keys and signatures")},
+    { 508, "check-sigs",256, N_("check key signatures")},
+    { 515, "fingerprint", 256, N_("list keys and fingerprints")},
+    { 558, "list-secret-keys", 256, N_("list secret keys")},
   #ifdef IS_G10
-    { 503, "gen-key",   0, N_("generate a new key pair")},
-    { 554, "add-key",   0, N_("add a subkey to a key pair")},
-    { 506, "sign-key"  ,0, N_("make a signature on a key in the keyring")},
-    { 505, "delete-key",0, N_("remove key from the public keyring")},
-    { 524, "edit-key"  ,0, N_("edit a key signature")},
-    { 525, "change-passphrase", 0, N_("change the passphrase of your secret keyring")},
-    { 542, "gen-revoke",0, N_("generate a revocation certificate")},
+    { 503, "gen-key",   256, N_("generate a new key pair")},
+    { 505, "delete-key",256, N_("remove key from the public keyring")},
+    { 524, "edit-key"  ,256, N_("sign or edit a key")},
+    { 542, "gen-revoke",256, N_("generate a revocation certificate")},
   #endif
-    { 537, "export"          , 0, N_("export keys") },
-    { 563, "export-secret-keys" , 0, "@" },
+    { 537, "export"          , 256, N_("export keys") },
+    { 563, "export-secret-keys" , 256, "@" },
     { 565, "do-not-export-rsa", 0, "@" },
-    { 530, "import",      0     , N_("import/merge keys")},
-    { 521, "list-packets",0,N_("list only the sequence of packets")},
+    { 530, "import",      256     , N_("import/merge keys")},
+    { 521, "list-packets",256,N_("list only the sequence of packets")},
   #ifdef IS_G10MAINT
-    { 564, "list-ownertrust", 0, "list the ownertrust values"},
-    { 546, "dearmor", 0, N_("De-Armor a file or stdin") },
-    { 547, "enarmor", 0, N_("En-Armor a file or stdin") },
-    { 555, "print-md" , 0, N_("|algo [files]|print message digests")},
-    { 516, "print-mds" , 0, N_("print all message digests")},
-    { 513, "gen-prime" , 0, "@" },
-    { 548, "gen-random" , 0, "@" },
+    { 564, "export-ownertrust", 256, N_("export the ownertrust values")},
+    { 525, "import-ownertrust", 256 , N_("import ownertrust values")},
+    { 567, "check-trustdb",0 , N_("|[NAMES]|check the trust database")},
+    { 546, "dearmor", 256, N_("De-Armor a file or stdin") },
+    { 547, "enarmor", 256, N_("En-Armor a file or stdin") },
+    { 555, "print-md" , 256, N_("|algo [files]|print message digests")},
+    { 516, "print-mds" , 256, N_("print all message digests")},
+    #ifdef MAINTAINER_OPTIONS
+    { 513, "gen-prime" , 256, "@" },
+    { 548, "gen-random" , 256, "@" },
+    #endif
   #endif
 
     { 301, NULL, 0, N_("@\nOptions:\n ") },
@@ -140,6 +142,7 @@ static ARGPARSE_OPTS opts[] = {
   #ifdef IS_G10MAINT
     { 514, "test"      , 0, "@" },
     { 564, "list-ownertrust",0 , "@"},
+    { 567, "check-trustdb",0 , "@"},
     { 531, "list-trustdb",0 , "@"},
     { 533, "list-trust-path",0, "@"},
   #endif
@@ -148,6 +151,7 @@ static ARGPARSE_OPTS opts[] = {
     { 504, "delete-secret-key",0, "@" },
     { 524, "edit-sig"  ,0, "@"}, /* alias for edit-key */
     { 523, "passphrase-fd",1, "@" },
+    { 506, "sign-key"  ,256, "@" }, /* alias for edit-key */
   #endif
     { 532, "quick-random", 0, "@"},
     { 526, "no-verbose", 0, "@"},
@@ -165,20 +169,22 @@ static ARGPARSE_OPTS opts[] = {
     { 508, "check-sig",0, "@" }, /* alias */
     { 553, "skip-verify",0, "@" },
     { 557, "compress-keys",0, "@"},
+    { 566, "compress-sigs",0, "@"},
     { 559, "always-trust", 0, "@"},
     { 562, "emulate-checksum-bug", 0, "@"},
-
+    { 554, "run-as-shm-coprocess", 4, "@" },
+     /* 568 unused */
 {0} };
 
 
 enum cmd_values { aNull = 0,
     aSym, aStore, aEncr, aKeygen, aSign, aSignEncr,
     aSignKey, aClearsign, aListPackets, aEditKey, aDeleteKey, aDeleteSecretKey,
-    aKMode, aKModeC, aChangePass, aImport, aVerify, aDecrypt, aListKeys,
-    aListSigs, aKeyadd, aListSecretKeys,
-    aExport, aExportSecret,
+    aKMode, aKModeC,  aImport, aVerify, aDecrypt, aListKeys,
+    aListSigs, aListSecretKeys, aExport, aExportSecret,
     aCheckKeys, aGenRevoke, aPrimegen, aPrintMD, aPrintMDs,
-    aListTrustDB, aListTrustPath, aListOwnerTrust,
+    aCheckTrustDB, aListTrustDB, aListTrustPath,
+    aExportOwnerTrust, aImportOwnerTrust,
     aDeArmor, aEnArmor, aGenRandom,
 aTest };
 
@@ -191,7 +197,6 @@ static void set_cmd( enum cmd_values *ret_cmd,
 #ifdef IS_G10MAINT
 static void print_hex( byte *p, size_t n );
 static void print_mds( const char *fname, int algo );
-static void do_test(int);
 #endif
 
 const char *
@@ -357,24 +362,6 @@ set_cmd( enum cmd_values *ret_cmd, enum cmd_values new_cmd )
 
 
 
-static void
-check_opts(void)
-{
-    if( !opt.def_cipher_algo || check_cipher_algo(opt.def_cipher_algo) )
-	log_error(_("selected cipher algorithm is invalid\n"));
-    if( opt.def_digest_algo && check_digest_algo(opt.def_digest_algo) )
-	log_error(_("selected digest algorithm is invalid\n"));
-    if( opt.def_compress_algo < 1 || opt.def_compress_algo > 2 )
-	log_error(_("compress algorithm must be in range %d..%d\n"), 1, 2);
-    if( opt.completes_needed < 1 )
-	log_error(_("completes-needed must be greater than 0\n"));
-    if( opt.marginals_needed < 2 )
-	log_error(_("marginals-needed must be greater than 1\n"));
-}
-
-
-
-
 int
 main( int argc, char **argv )
 {
@@ -400,6 +387,9 @@ main( int argc, char **argv )
     const char *trustdb_name = NULL;
     char *def_cipher_string = NULL;
     char *def_digest_string = NULL;
+  #ifdef USE_SHM_COPROCESSING
+    ulong requested_shm_size=0;
+  #endif
 
     trap_unaligned();
   #ifdef IS_G10MAINT
@@ -413,12 +403,13 @@ main( int argc, char **argv )
      */
     log_set_name("gpg");
     secure_random_alloc(); /* put random number into secure memory */
+    disable_core_dumps();
     init_signals();
   #endif
     i18n_init();
     opt.compress = -1; /* defaults to standard compress level */
     /* fixme: set the next two to zero and decide where used */
-    opt.def_cipher_algo = DEFAULT_CIPHER_ALGO;
+    opt.def_cipher_algo = 0;
     opt.def_digest_algo = 0;
     opt.def_compress_algo = 2;
     opt.completes_needed = 1;
@@ -515,7 +506,6 @@ main( int argc, char **argv )
 	  case 507: set_cmd( &cmd, aStore); break;
 	  case 523: set_passphrase_fd( pargs.r.ret_int ); break;
 	  case 524: set_cmd( &cmd, aEditKey); break;
-	  case 525: set_cmd( &cmd, aChangePass); break;
 	  case 527: def_cipher_string = m_strdup(pargs.r.ret_str); break;
 	  case 529: def_digest_string = m_strdup(pargs.r.ret_str); break;
 	  case 539: set_cmd( &cmd, aClearsign); break;
@@ -529,17 +519,21 @@ main( int argc, char **argv )
 	#endif /* !IS_G10 */
 
 	#ifdef IS_G10MAINT
-	  case 513: set_cmd( &cmd, aPrimegen); break;
-	  case 514: set_cmd( &cmd, aTest); break;
+	  #ifdef MAINTAINER_OPTIONS
+	    case 513: set_cmd( &cmd, aPrimegen); break;
+	    case 514: set_cmd( &cmd, aTest); break;
+	    case 548: set_cmd( &cmd, aGenRandom); break;
+	  #endif
 	  case 516: set_cmd( &cmd, aPrintMDs); break;
 	  case 531: set_cmd( &cmd, aListTrustDB); break;
+	  case 567: set_cmd( &cmd, aCheckTrustDB); break;
 	  case 533: set_cmd( &cmd, aListTrustPath); break;
 	  case 540: break; /* dummy */
 	  case 546: set_cmd( &cmd, aDeArmor); break;
 	  case 547: set_cmd( &cmd, aEnArmor); break;
-	  case 548: set_cmd( &cmd, aGenRandom); break;
 	  case 555: set_cmd( &cmd, aPrintMD); break;
-	  case 564: set_cmd( &cmd, aListOwnerTrust); break;
+	  case 564: set_cmd( &cmd, aExportOwnerTrust); break;
+	  case 525: set_cmd( &cmd, aImportOwnerTrust); break;
 	#endif /* IS_G10MAINT */
 
 	  case 'o': opt.outfile = pargs.r.ret_str; break;
@@ -555,7 +549,7 @@ main( int argc, char **argv )
 	  case 510: opt.debug |= pargs.r.ret_ulong; break;
 	  case 511: opt.debug = ~0; break;
 	  case 512: set_status_fd( pargs.r.ret_int ); break;
-	  case 515: opt.fingerprint = 1; break;
+	  case 515: opt.fingerprint++; break;
 	  case 517: append_to_strlist( &sec_nrings, pargs.r.ret_str); break;
 	  case 518:
 	    /* config files may not be nested (silently ignore them) */
@@ -586,16 +580,24 @@ main( int argc, char **argv )
 	  case 551: set_cmd( &cmd, aListKeys); break;
 	  case 552: set_cmd( &cmd, aListSigs); break;
 	  case 553: opt.skip_verify=1; break;
-	  case 554: set_cmd( &cmd, aKeyadd); break;
 	  case 556: opt.def_compress_algo = pargs.r.ret_int; break;
 	  case 557: opt.compress_keys = 1; break;
 	  case 558: set_cmd( &cmd, aListSecretKeys); break;
 	  case 559: opt.always_trust = 1; break;
 	  case 560: register_cipher_extension(pargs.r.ret_str); break;
-	  case 561: opt.rfc1991 = 1; break;
-	  case 562: opt.emulate_bugs |= 1; break;
+	  case 561: opt.rfc1991 = 1; opt.no_comment = 1; break;
+	  case 562: opt.emulate_bugs |= EMUBUG_GPGCHKSUM; break;
 	  case 563: set_cmd( &cmd, aExportSecret); break;
 	  case 565: opt.do_not_export_rsa = 1; break;
+	  case 566: opt.compress_sigs = 1; break;
+	  case 554:
+	  #ifdef USE_SHM_COPROCESSING
+	    opt.shm_coprocess = 1;
+	    requested_shm_size = pargs.r.ret_ulong;
+	  #else
+	    log_error("shared memory coprocessing is not available\n");
+	  #endif
+	    break;
 	  default : errors++; pargs.err = configfp? 1:2; break;
 	}
     }
@@ -614,6 +616,15 @@ main( int argc, char **argv )
 	tty_printf("%s\n", strusage(15) );
     }
 
+  #ifdef USE_SHM_COPROCESSING
+    if( opt.shm_coprocess ) {
+      #ifdef IS_G10
+	init_shm_coprocessing(requested_shm_size, 1 );
+      #else
+	init_shm_coprocessing(requested_shm_size, 0 );
+      #endif
+    }
+  #endif
   #ifdef IS_G10
     /* initialize the secure memory. */
     secmem_init( 16384 );
@@ -621,7 +632,6 @@ main( int argc, char **argv )
     /* Okay, we are now working under our real uid */
   #endif
 
-    /*write_status( STATUS_ENTER );*/
 
     set_debug();
 
@@ -630,12 +640,22 @@ main( int argc, char **argv )
     if( def_cipher_string ) {
 	opt.def_cipher_algo = string_to_cipher_algo(def_cipher_string);
 	m_free(def_cipher_string); def_cipher_string = NULL;
+	if( check_cipher_algo(opt.def_cipher_algo) )
+	    log_error(_("selected cipher algorithm is invalid\n"));
     }
     if( def_digest_string ) {
 	opt.def_digest_algo = string_to_digest_algo(def_digest_string);
 	m_free(def_digest_string); def_digest_string = NULL;
+	if( check_digest_algo(opt.def_digest_algo) )
+	    log_error(_("selected digest algorithm is invalid\n"));
     }
-    check_opts();
+    if( opt.def_compress_algo < 1 || opt.def_compress_algo > 2 )
+	log_error(_("compress algorithm must be in range %d..%d\n"), 1, 2);
+    if( opt.completes_needed < 1 )
+	log_error(_("completes-needed must be greater than 0\n"));
+    if( opt.marginals_needed < 2 )
+	log_error(_("marginals-needed must be greater than 1\n"));
+
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
@@ -712,7 +732,7 @@ main( int argc, char **argv )
 	if( opt.with_colons ) /* need this to list the trust */
 	    rc = init_trustdb(1, trustdb_name );
 	break;
-      case aListOwnerTrust: rc = init_trustdb( 0, trustdb_name ); break;
+      case aExportOwnerTrust: rc = init_trustdb( 0, trustdb_name ); break;
       case aListTrustDB: rc = init_trustdb( argc? 1:0, trustdb_name ); break;
       default: rc = init_trustdb(1, trustdb_name ); break;
     }
@@ -725,16 +745,16 @@ main( int argc, char **argv )
 	if( argc > 1 )
 	    wrong_args(_("--store [filename]"));
 	if( (rc = encode_store(fname)) )
-	    log_error("%s: store failed: %s\n",
-				 print_fname_stdin(fname), g10_errstr(rc) );
+	    log_error_f( print_fname_stdin(fname),
+			"store failed: %s\n", g10_errstr(rc) );
 	break;
     #ifdef IS_G10
       case aSym: /* encrypt the given file only with the symmetric cipher */
 	if( argc > 1 )
 	    wrong_args(_("--symmetric [filename]"));
 	if( (rc = encode_symmetric(fname)) )
-	    log_error("%s: symmetric encryption failed: %s\n",
-			    print_fname_stdin(fname), g10_errstr(rc) );
+	    log_error_f(print_fname_stdin(fname),
+			"symmetric encryption failed: %s\n",g10_errstr(rc) );
 	break;
 
       case aEncr: /* encrypt the given file */
@@ -798,19 +818,10 @@ main( int argc, char **argv )
 
 
       case aSignKey: /* sign the key given as argument */
-	if( argc != 1 )
-	    wrong_args(_("--sign-key username"));
-	/* note: fname is the user id! */
-	if( (rc = sign_key(fname, locusr)) )
-	    log_error("%s: sign key failed: %s\n", print_fname_stdin(fname), g10_errstr(rc) );
-	break;
-
       case aEditKey: /* Edit a key signature */
 	if( argc != 1 )
 	    wrong_args(_("--edit-key username"));
-	/* note: fname is the user id! */
-	if( (rc = edit_keysigs(fname)) )
-	    log_error("%s: edit signature failed: %s\n", print_fname_stdin(fname), g10_errstr(rc) );
+	keyedit_menu(fname, locusr );
 	break;
 
       case aDeleteSecretKey:
@@ -824,14 +835,6 @@ main( int argc, char **argv )
 	    log_error("%s: delete key failed: %s\n", print_fname_stdin(fname), g10_errstr(rc) );
 	break;
 
-      case aChangePass: /* Change the passphrase */
-	if( argc > 1 ) /* no arg: use default, 1 arg use this one */
-	    wrong_args(_("--change-passphrase [username]"));
-	/* note: fname is the user id! */
-	if( (rc = change_passphrase(fname)) )
-	    log_error("%s: change passphrase failed: %s\n", print_fname_stdin(fname),
-						       g10_errstr(rc) );
-	break;
       #endif /* IS_G10 */
 
       case aCheckKeys:
@@ -870,11 +873,6 @@ main( int argc, char **argv )
 	    wrong_args("--gen-key");
 	generate_keypair();
 	break;
-      case aKeyadd: /* add a subkey (interactive) */
-	if( argc != 1 )
-	    wrong_args("--add-key userid");
-	generate_subkeypair(*argv);
-	break;
     #endif
 
       case aImport:
@@ -933,6 +931,7 @@ main( int argc, char **argv )
 	break;
 
 
+     #ifdef MAINTAINER_OPTIONS
       case aPrimegen:
 	if( argc == 1 ) {
 	    mpi_print( stdout, generate_public_prime( atoi(argv[0]) ), 1);
@@ -960,23 +959,27 @@ main( int argc, char **argv )
 	else
 	    usage(1);
 	break;
+      #endif /* MAINTAINER OPTIONS */
 
+      #ifdef MAINTAINER_OPTIONS
       case aGenRandom:
 	if( argc < 1 || argc > 2 )
 	    wrong_args("--gen-random level [hex]");
 	{
 	    int level = atoi(*argv);
 	    for(;;) {
-		int c = get_random_byte(level);
+		byte *p = get_random_bits( 8, level, 0);
 		if( argc == 1 ) {
-		    printf("%02x", c );
+		    printf("%02x", *p );
 		    fflush(stdout);
 		}
 		else
 		    putchar(c&0xff);
+		m_free(p);
 	    }
 	}
 	break;
+      #endif /* MAINTAINER OPTIONS */
 
       case aPrintMD:
 	if( argc < 1)
@@ -1007,7 +1010,9 @@ main( int argc, char **argv )
 	}
 	break;
 
+     #ifdef MAINTAINER_OPTIONS
       case aTest: do_test( argc? atoi(*argv): 1 ); break;
+      #endif /* MAINTAINER OPTIONS */
 
       case aListTrustDB:
 	if( !argc )
@@ -1018,16 +1023,31 @@ main( int argc, char **argv )
 	}
 	break;
 
+      case aCheckTrustDB:
+	if( !argc )
+	    check_trustdb(NULL);
+	else {
+	    for( ; argc; argc--, argv++ )
+		check_trustdb( *argv );
+	}
+	break;
+
       case aListTrustPath:
 	if( argc != 2 )
 	    wrong_args("--list-trust-path [-- -]<maxdepth> <username>");
 	list_trust_path( atoi(*argv), argv[1] );
 	break;
 
-      case aListOwnerTrust:
+      case aExportOwnerTrust:
 	if( argc )
-	    wrong_args("--list-ownertrust");
-	list_ownertrust();
+	    wrong_args("--export-ownertrust");
+	export_ownertrust();
+	break;
+
+      case aImportOwnerTrust:
+	if( argc > 1 )
+	    wrong_args("--import-ownertrust [file]");
+	import_ownertrust( argc? *argv:NULL );
 	break;
 
      #endif /* IS_G10MAINT */
@@ -1085,7 +1105,7 @@ do_not_use_RSA()
 
     if( !did_rsa_note ) {
 	did_rsa_note = 1;
-	log_info(_("RSA keys are depreciated; please consider "
+	log_info(_("RSA keys are deprecated; please consider "
 		   "creating a new key and use this key in the future\n"));
     }
 }
@@ -1194,10 +1214,12 @@ print_mds( const char *fname, int algo )
 
 
 
+#ifdef MAINTAINER_OPTIONS
 static void
 do_test(int times)
 {
     m_check(NULL);
 }
+#endif /* MAINTAINER OPTIONS */
 #endif /* IS_G10MAINT */
 
