@@ -464,6 +464,20 @@ get_pubkey_byname( PKT_public_key *pk, const char *name )
 }
 
 
+/****************
+ * Search for a key with the given fingerprint.
+ */
+int
+get_pubkey_byfprint( PKT_public_key *pk, const byte *fprint, size_t fprint_len)
+{
+    int rc;
+
+    if( fprint_len == 20 || fprint_len == 16 )
+	rc = lookup( pk, fprint_len, NULL, fprint, NULL );
+    else
+	rc = G10ERR_GENERAL; /* Oops */
+    return rc;
+}
 
 /****************
  * Search for a key with the given fingerprint and return the
@@ -671,6 +685,8 @@ lookup( PKT_public_key *pk, int mode,  u32 *keyid,
     KBNODE keyblock = NULL;
     KBPOS kbpos;
     int oldmode = set_packet_list_mode(0);
+    byte namehash[20];
+    int use_namehash=0;
 
     rc = enum_keyblocks( 0, &kbpos, &keyblock );
     if( rc ) {
@@ -705,6 +721,10 @@ lookup( PKT_public_key *pk, int mode,  u32 *keyid,
 			u32 aki[2];
 			keyid_from_pk( kk->pkt->pkt.public_key, aki );
 			cache_user_id( k->pkt->pkt.user_id, aki );
+			rmd160_hash_buffer( namehash,
+					    k->pkt->pkt.user_id->name,
+					    k->pkt->pkt.user_id->len );
+			use_namehash = 1;
 			k = kk;
 			break;
 		    }
@@ -758,7 +778,7 @@ lookup( PKT_public_key *pk, int mode,  u32 *keyid,
 		    else if( mode == 16 || mode == 20 ) {
 			size_t an;
 			byte *afp = fingerprint_from_pk(
-					k->pkt->pkt.public_key, &an );
+					k->pkt->pkt.public_key, NULL, &an );
 
 			if( DBG_CACHE ) {
 			    u32 aki[2];
@@ -785,7 +805,8 @@ lookup( PKT_public_key *pk, int mode,  u32 *keyid,
 	if( k ) { /* found */
 	    assert(    k->pkt->pkttype == PKT_PUBLIC_KEY
 		    || k->pkt->pkttype == PKT_PUBLIC_SUBKEY );
-	    copy_public_key( pk, k->pkt->pkt.public_key );
+	    copy_public_key_new_namehash( pk, k->pkt->pkt.public_key,
+					  use_namehash? namehash:NULL);
 	    add_stuff_from_selfsig( keyblock, k );
 	    if( ret_keyblock ) {
 		*ret_keyblock = keyblock;
@@ -897,7 +918,7 @@ lookup_sk( PKT_secret_key *sk, int mode,  u32 *keyid, const char *name )
 		    else if( mode == 16 || mode == 20 ) {
 			size_t an;
 			byte *afp = fingerprint_from_sk(
-					k->pkt->pkt.secret_key, &an );
+					k->pkt->pkt.secret_key, NULL, &an );
 			if( an == mode && !memcmp( afp, name, an)
 			    && ( !sk->pubkey_algo
 				 || sk->pubkey_algo
@@ -936,7 +957,7 @@ lookup_sk( PKT_secret_key *sk, int mode,  u32 *keyid, const char *name )
 
 
 /****************
- * Enumerate all secret keys.  Caller must use these procedure:
+ * Enumerate all primary secret keys.  Caller must use these procedure:
  *  1) create a void pointer and initialize it to NULL
  *  2) pass this void pointer by reference to this function
  *     and provide space for the secret key (pass a buffer for sk)
@@ -983,8 +1004,7 @@ enum_secret_keys( void **context, PKT_secret_key *sk )
 	while( (rc=parse_packet(c->iobuf, &pkt)) != -1 ) {
 	    if( rc )
 		; /* e.g. unknown packet */
-	    else if( pkt.pkttype == PKT_SECRET_KEY
-		    || pkt.pkttype == PKT_SECRET_SUBKEY ) {
+	    else if( pkt.pkttype == PKT_SECRET_KEY ) {
 		copy_secret_key( sk, pkt.pkt.secret_key );
 		set_packet_list_mode(save_mode);
 		return 0; /* found */
