@@ -381,6 +381,17 @@ import_one( const char *fname, KBNODE keyblock )
 	else
 	    log_info_f(fname, _("key %08lX: not changed\n"), (ulong)keyid[1] );
     }
+    if( !rc ) {
+	rc = query_trust_record( pk_orig );
+	if( rc && rc != -1 )
+	    log_error("trustdb error: %s\n", g10_errstr(rc) );
+	else if( rc == -1 ) {
+	    rc = insert_trust_record( pk_orig );
+	    if( rc )
+		log_error("key %08lX: trustdb insert failed: %s\n",
+					(ulong)keyid[1], g10_errstr(rc) );
+	}
+    }
 
   leave:
     release_kbnode( keyblock_orig );
@@ -840,7 +851,7 @@ merge_sigs( KBNODE dst, KBNODE src, int *n_sigs,
 	 * We add a clone to the original keyblock, because this
 	 * one is released first */
 	n2 = clone_kbnode(n);
-	insert_kbnode( dst, n2, PKT_USER_ID );
+	insert_kbnode( dst, n2, PKT_SIGNATURE );
 	n2->flag |= 1;
 	n->flag |= 1;
 	++*n_sigs;
