@@ -189,6 +189,43 @@ keyid_from_pk( PKT_public_key *pk, u32 *keyid )
 }
 
 
+/****************
+ * Get the keyid from the fingerprint.	This function is simple for most
+ * keys, but has to do a keylookup for old stayle keys.
+ */
+u32
+keyid_from_fingerprint( const byte *fprint, size_t fprint_len, u32 *keyid )
+{
+    u32 dummy_keyid[2];
+
+    if( !keyid )
+	keyid = dummy_keyid;
+
+    if( fprint_len != 20 ) {
+	/* This is special as we have to lookup the key first */
+	PKT_public_key pk;
+	int rc;
+
+	memset( &pk, 0, sizeof pk );
+	rc = get_pubkey_byfprint( &pk, fprint, fprint_len );
+	if( rc ) {
+	    log_error("Oops: keyid_from_fingerprint: no pubkey\n");
+	    keyid[0] = 0;
+	    keyid[1] = 0;
+	}
+	else
+	    keyid_from_pk( &pk, keyid );
+    }
+    else {
+	const byte *dp = fprint;
+	keyid[0] = dp[12] << 24 | dp[13] << 16 | dp[14] << 8 | dp[15] ;
+	keyid[1] = dp[16] << 24 | dp[17] << 16 | dp[18] << 8 | dp[19] ;
+    }
+
+    return keyid[1];
+}
+
+
 u32
 keyid_from_sig( PKT_signature *sig, u32 *keyid )
 {
@@ -259,16 +296,47 @@ datestr_from_sig( PKT_signature *sig )
 }
 
 
+const char *
+expirestr_from_pk( PKT_public_key *pk )
+{
+    static char buffer[11+5];
+    struct tm *tp;
+    time_t atime;
+
+    if( !pk->valid_days )
+	return "never     ";
+    atime = add_days_to_timestamp( pk->timestamp, pk->valid_days );
+    tp = gmtime( &atime );
+    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
+    return buffer;
+}
+
+const char *
+expirestr_from_sk( PKT_secret_key *sk )
+{
+    static char buffer[11+5];
+    struct tm *tp;
+    time_t atime;
+
+    if( !sk->valid_days )
+	return "never     ";
+    atime = add_days_to_timestamp( sk->timestamp, sk->valid_days );
+    tp = gmtime( &atime );
+    sprintf(buffer,"%04d-%02d-%02d", 1900+tp->tm_year, tp->tm_mon+1, tp->tm_mday );
+    return buffer;
+}
+
+
 /**************** .
  * Return a byte array with the fingerprint for the given PK/SK
  * The length of the array is returned in ret_len. Caller must free
- * the array.
+ * the array or provide array as buffer of length MAX_FINGERPRINT_LEN
  */
 
 byte *
-fingerprint_from_pk( PKT_public_key *pk, size_t *ret_len )
+fingerprint_from_pk( PKT_public_key *pk, byte *array, size_t *ret_len )
 {
-    byte *p, *buf, *array;
+    byte *p, *buf;
     const char *dp;
     size_t len;
     unsigned n;
@@ -287,7 +355,8 @@ fingerprint_from_pk( PKT_public_key *pk, size_t *ret_len )
 	    m_free(buf);
 	}
 	md_final(md);
-	array = m_alloc( 16 );
+	if( !array )
+	    array = m_alloc( 16 );
 	len = 16;
 	memcpy(array, md_read(md, DIGEST_ALGO_MD5), 16 );
 	md_close(md);
@@ -297,7 +366,9 @@ fingerprint_from_pk( PKT_public_key *pk, size_t *ret_len )
 	md = do_fingerprint_md(pk);
 	dp = md_read( md, 0 );
 	len = md_digest_length( md_get_algo( md ) );
-	array = m_alloc( len );
+	assert( len <= MAX_FINGERPRINT_LEN );
+	if( !array )
+	    array = m_alloc( len );
 	memcpy(array, dp, len );
 	md_close(md);
     }
@@ -307,9 +378,9 @@ fingerprint_from_pk( PKT_public_key *pk, size_t *ret_len )
 }
 
 byte *
-fingerprint_from_sk( PKT_secret_key *sk, size_t *ret_len )
+fingerprint_from_sk( PKT_secret_key *sk, byte *array, size_t *ret_len )
 {
-    byte *p, *buf, *array;
+    byte *p, *buf;
     const char *dp;
     size_t len;
     unsigned n;
@@ -328,7 +399,8 @@ fingerprint_from_sk( PKT_secret_key *sk, size_t *ret_len )
 	    m_free(buf);
 	}
 	md_final(md);
-	array = m_alloc( 16 );
+	if( !array )
+	    array = m_alloc( 16 );
 	len = 16;
 	memcpy(array, md_read(md, DIGEST_ALGO_MD5), 16 );
 	md_close(md);
@@ -338,7 +410,9 @@ fingerprint_from_sk( PKT_secret_key *sk, size_t *ret_len )
 	md = do_fingerprint_md_sk(sk);
 	dp = md_read( md, 0 );
 	len = md_digest_length( md_get_algo( md ) );
-	array = m_alloc( len );
+	assert( len <= MAX_FINGERPRINT_LEN );
+	if( !array )
+	    array = m_alloc( len );
 	memcpy(array, dp, len );
 	md_close(md);
     }
