@@ -47,6 +47,7 @@ void do_not_use_RSA(void);
 
 /*-- misc.c --*/
 void trap_unaligned(void);
+void disable_core_dumps(void);
 u16 checksum_u16( unsigned n );
 u16 checksum( byte *p, unsigned n );
 u16 checksum_mpi( MPI a );
@@ -71,14 +72,13 @@ int clearsign_file( const char *fname, STRLIST locusr, const char *outfile );
 int check_key_signature( KBNODE root, KBNODE node, int *is_selfsig );
 
 /*-- keyedit.c --*/
-int sign_key( const char *username, STRLIST locusr );
-int edit_keysigs( const char *username );
 int delete_key( const char *username, int secure );
-int change_passphrase( const char *username );
+void keyedit_menu( const char *username, STRLIST locusr );
 
 /*-- keygen.c --*/
 void generate_keypair(void);
-void generate_subkeypair(const char *userid);
+int keygen_add_std_prefs( PKT_signature *sig, void *opaque );
+int generate_subkeypair( KBNODE pub_keyblock, KBNODE sec_keyblock );
 
 /*-- openfile.c --*/
 int overwrite_filep( const char *fname );
@@ -119,9 +119,11 @@ int verify_signatures( int nfiles, char **files );
 int decrypt_message( const char *filename );
 
 /*-- plaintext.c --*/
-int hash_datafiles( MD_HANDLE md, STRLIST files, int textmode );
+int hash_datafiles( MD_HANDLE md, STRLIST files, const char *sigfilename,
+		    int textmode );
 
 /*-- signal.c --*/
 void init_signals(void);
+void pause_on_sigusr( int which );
 
 #endif /*G10_MAIN_H*/
