@@ -51,6 +51,7 @@ typedef struct {
     int sigs_only;   /* process only signatures and reject all other stuff */
     int encrypt_only; /* process only encrytion messages */
     STRLIST signed_data;
+    const char *sigfilename;
     DEK *dek;
     int last_was_session_key;
     KBNODE list;   /* the current list of packets */
@@ -170,6 +171,7 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     PKT_pubkey_enc *enc;
     int result = 0;
 
+    /* check whether the secret key is available and store in this case */
     c->last_was_session_key = 1;
     enc = pkt->pkt.pubkey_enc;
     /*printf("enc: encrypted by a pubkey with keyid %08lX\n", enc->keyid[1] );*/
@@ -207,7 +209,9 @@ proc_encrypted( CTX c, PACKET *pkt )
     /*printf("dat: %sencrypted data\n", c->dek?"":"conventional ");*/
     if( !c->dek && !c->last_was_session_key ) {
 	/* assume this is old conventional encrypted data */
-	c->dek = passphrase_to_dek( NULL, opt.def_cipher_algo, NULL, 0 );
+	c->dek = passphrase_to_dek( NULL,
+		    opt.def_cipher_algo ? opt.def_cipher_algo
+					: DEFAULT_CIPHER_ALGO, NULL, 0 );
     }
     else if( !c->dek )
 	result = G10ERR_NO_SECKEY;
@@ -222,6 +226,8 @@ proc_encrypted( CTX c, PACKET *pkt )
     }
     else {
 	log_error(_("decryption failed: %s\n"), g10_errstr(result));
+	/* FIXME: if this is secret key not available, try with
+	 * other keys */
     }
     free_packet(pkt);
     c->last_was_session_key = 0;
@@ -262,7 +268,7 @@ proc_plaintext( CTX c, PACKET *pkt )
 		clearsig = 1;
 	}
     }
-    if( !any ) { /* no onepass sig packet: enable all algos */
+    if( !any ) { /* no onepass sig packet: enable all standard algos */
 	md_enable( c->mfx.md, DIGEST_ALGO_RMD160 );
 	md_enable( c->mfx.md, DIGEST_ALGO_SHA1 );
 	md_enable( c->mfx.md, DIGEST_ALGO_MD5 );
@@ -273,13 +279,13 @@ proc_plaintext( CTX c, PACKET *pkt )
 	    m_check( c->mfx.md->list );
     }
     rc = handle_plaintext( pt, &c->mfx, c->sigs_only, clearsig );
+    if( rc == G10ERR_CREATE_FILE && !c->sigs_only) {
+	/* can't write output but we hash it anyway to
+	 * check the signature */
+	rc = handle_plaintext( pt, &c->mfx, 1, clearsig );
+    }
     if( rc )
 	log_error( "handle plaintext failed: %s\n", g10_errstr(rc));
-    if( c->mfx.md ) {
-	m_check(c->mfx.md);
-	if( c->mfx.md->list )
-	    m_check( c->mfx.md->list );
-    }
     free_packet(pkt);
     c->last_was_session_key = 0;
 }
@@ -288,7 +294,8 @@ proc_plaintext( CTX c, PACKET *pkt )
 static int
 proc_compressed_cb( IOBUF a, void *info )
 {
-    return proc_signature_packets( a, ((CTX)info)->signed_data );
+    return proc_signature_packets( a, ((CTX)info)->signed_data,
+				      ((CTX)info)->sigfilename );
 }
 
 static int
@@ -405,8 +412,8 @@ print_fingerprint( PKT_public_key *pk, PKT_secret_key *sk )
     byte *array, *p;
     size_t i, n;
 
-    p = array = sk? fingerprint_from_sk( sk, &n )
-		   : fingerprint_from_pk( pk, &n );
+    p = array = sk? fingerprint_from_sk( sk, NULL, &n )
+		   : fingerprint_from_pk( pk, NULL, &n );
     if( opt.with_colons ) {
 	printf("fpr:::::::::");
 	for(i=0; i < n ; i++, p++ )
@@ -518,6 +525,8 @@ list_node( CTX c, KBNODE node )
 	}
 	if( !any )
 	    putchar('\n');
+	if( !mainkey && opt.fingerprint > 1 )
+	    print_fingerprint( pk, NULL );
     }
     else if( (mainkey = (node->pkt->pkttype == PKT_SECRET_KEY) )
 	     || node->pkt->pkttype == PKT_SECRET_SUBKEY ) {
@@ -581,6 +590,8 @@ list_node( CTX c, KBNODE node )
 	}
 	if( !any )
 	    putchar('\n');
+	if( !mainkey && opt.fingerprint > 1 )
+	    print_fingerprint( NULL, sk );
     }
     else if( node->pkt->pkttype == PKT_SIGNATURE  ) {
 	PKT_signature *sig = node->pkt->pkt.signature;
@@ -665,12 +676,13 @@ proc_packets( IOBUF a )
 }
 
 int
-proc_signature_packets( IOBUF a, STRLIST signedfiles )
+proc_signature_packets( IOBUF a, STRLIST signedfiles, const char *sigfilename )
 {
     CTX c = m_alloc_clear( sizeof *c );
     int rc;
     c->sigs_only = 1;
     c->signed_data = signedfiles;
+    c->sigfilename = sigfilename;
     rc = do_proc_packets( c, a );
     m_free( c );
     return rc;
@@ -700,7 +712,9 @@ do_proc_packets( CTX c, IOBUF a )
     while( (rc=parse_packet(a, pkt)) != -1 ) {
 	/* cleanup if we have an illegal data structure */
 	if( c->dek && pkt->pkttype != PKT_ENCRYPTED ) {
-	    log_error("oops: valid pubkey enc packet not followed by data\n");
+	    /* FIXME: do we need to ave it in case we have no secret
+	     * key for one of the next reciepents- we should check it
+	     * here. */
 	    m_free(c->dek); c->dek = NULL; /* burn it */
 	}
 
@@ -881,8 +895,8 @@ proc_tree( CTX c, KBNODE node )
 	    }
 	    /* ask for file and hash it */
 	    if( c->sigs_only )
-		rc = hash_datafiles( c->mfx.md, c->signed_data,
-			    n1->pkt->pkt.onepass_sig->sig_class == 0x01 );
+		rc = hash_datafiles( c->mfx.md, c->signed_data, c->sigfilename,
+			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
 	    else
 		rc = ask_for_detached_datafile( &c->mfx,
 					    iobuf_get_fname(c->iobuf));
@@ -902,7 +916,7 @@ proc_tree( CTX c, KBNODE node )
 	    free_md_filter_context( &c->mfx );
 	    c->mfx.md = md_open(sig->digest_algo, 0);
 	    if( c->sigs_only )
-		rc = hash_datafiles( c->mfx.md, c->signed_data,
+		rc = hash_datafiles( c->mfx.md, c->signed_data, c->sigfilename,
 				     sig->sig_class == 0x01 );
 	    else
 		rc = ask_for_detached_datafile( &c->mfx,
