@@ -47,12 +47,16 @@ struct {
     const char *homedir;
     int skip_verify;
     int compress_keys;
+    int compress_sigs;
     int always_trust;
     int rfc1991;
-    unsigned emulate_bugs; /* bug emulation flags */
+    unsigned emulate_bugs; /* bug emulation flags EMUBUG_xxxx */
+    int shm_coprocess;
 } opt;
 
 
+#define EMUBUG_GPGCHKSUM  1
+
 #define DBG_PACKET_VALUE  1	/* debug packet reading/writing */
 #define DBG_MPI_VALUE	  2	/* debug mpi details */
 #define DBG_CIPHER_VALUE  4	/* debug cipher handling */
