@@ -32,6 +32,7 @@
 #include "keydb.h"
 #include "main.h"
 #include "i18n.h"
+#include "status.h"
 
 static int pwfd = -1;
 static char *next_pw = NULL;
@@ -129,6 +130,12 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 	tty_printf("\n");
 	free_public_key( pk );
     }
+    else if( keyid && !next_pw ) {
+	char buf[20];
+	sprintf( buf, "%08lX%08lX", (ulong)keyid[0], (ulong)keyid[1] );
+	write_status_text( STATUS_NEED_PASSPHRASE, buf );
+    }
+
     if( next_pw ) {
 	pw = next_pw;
 	next_pw = NULL;
@@ -157,10 +164,11 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
     else if( opt.batch )
 	log_fatal("Can't query password in batchmode\n");
     else {
-	pw = tty_get_hidden("Enter pass phrase: " );
+	pw = cpr_get_hidden(N_("passphrase.enter"), _("Enter pass phrase: ") );
 	tty_kill_prompt();
-	if( mode == 2 ) {
-	    char *pw2 = tty_get_hidden("Repeat pass phrase: " );
+	if( mode == 2 && !cpr_enabled() ) {
+	    char *pw2 = cpr_get_hidden(N_("passphrase.repeat"),
+				       _("Repeat pass phrase: ") );
 	    tty_kill_prompt();
 	    if( strcmp(pw, pw2) ) {
 		m_free(pw2);
@@ -195,12 +203,38 @@ hash_passphrase( DEK *dek, char *pw, STRING2KEY *s2k, int create )
     assert( s2k->hash_algo );
     dek->keylen = 0;
     md = md_open( s2k->hash_algo, 1);
-    if( s2k->mode == 1 || s2k->mode == 4 ) {
+    if( s2k->mode == 1 || s2k->mode == 3 ) {
+	ulong count = 0;
+	int len = strlen(pw);
+	int len2 = len + 8;
+
 	if( create )
 	    randomize_buffer(s2k->salt, 8, 1);
-	md_write( md, s2k->salt, 8 );
+
+	if( s2k->mode == 3 ) {
+	    count = (16ul + (s2k->count & 15)) << ((s2k->count >> 4) + 6);
+	    log_debug("s2k iteration count=%lu\n", count );
+	}
+	for(;;) {
+	    md_write( md, s2k->salt, 8 );
+	    md_write( md, pw, len );
+	    if( count <= len2 )
+		break;
+	    count -= len2;
+	}
+	if( count ) {
+	    if( count < 8 )
+		md_write( md, s2k->salt, count );
+	    else {
+		md_write( md, s2k->salt, 8 );
+		count -= 8;
+		assert( count <= len );
+		md_write( md, pw, count );
+	    }
+	}
     }
-    md_write( md, pw, strlen(pw) );
+    else
+	md_write( md, pw, strlen(pw) );
     md_final( md );
     dek->keylen = cipher_get_keylen( dek->algo ) / 8;
     assert(dek->keylen > 0 && dek->keylen <= DIM(dek->key) );
