@@ -39,8 +39,8 @@
 /****************
  * Returns true if an ownertrust has changed.
  */
-static int
-query_ownertrust( ulong lid )
+int
+edit_ownertrust( ulong lid, int mode )
 {
     char *p;
     int rc;
@@ -49,7 +49,7 @@ query_ownertrust( ulong lid )
     PKT_public_key *pk ;
     int changed=0;
 
-    rc = keyid_from_trustdb( lid, keyid );
+    rc = keyid_from_lid( lid, keyid );
     if( rc ) {
 	log_error("ooops: can't get keyid for lid %lu\n", lid);
 	return 0;
@@ -63,14 +63,17 @@ query_ownertrust( ulong lid )
 	return 0;
     }
 
-    tty_printf(_("No ownertrust defined for %lu:\n"
-	       "%4u%c/%08lX %s \""), lid,
-	      nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
-	      (ulong)keyid[1], datestr_from_pk( pk ) );
-    p = get_user_id( keyid, &n );
-    tty_print_string( p, n ),
-    m_free(p);
-    tty_printf(_("\"\n\n"
+    if( !mode ) {
+	tty_printf(_("No owner trust defined for %lu:\n"
+		   "%4u%c/%08lX %s \""), lid,
+		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
+		  (ulong)keyid[1], datestr_from_pk( pk ) );
+	p = get_user_id( keyid, &n );
+	tty_print_string( p, n ),
+	m_free(p);
+	tty_printf("\"\n\n");
+    }
+    tty_printf(_(
 "Please decide how far you trust this user to correctly\n"
 "verify other users' keys (by looking at passports,\n"
 "checking fingerprints from different sources...)?\n\n"
@@ -81,17 +84,11 @@ query_ownertrust( ulong lid )
 " s = please show me more information\n\n") );
 
     for(;;) {
-	p = tty_get(_("Your decision? "));
+	p = cpr_get(N_("edit_ownertrust.value"),_("Your decision? "));
 	trim_spaces(p);
-	tty_kill_prompt();
+	cpr_kill_prompt();
 	if( *p && p[1] )
 	    ;
-	else if( *p == '?' ) {
-	    tty_printf(_(
-"It's up to you to assign a value here; this value will never be exported\n"
-"to any 3rd party.  We need it to implement the web-of-trust; it has nothing\n"
-"to do with the (implicitly created) web-of-certificates.\n"));
-	}
 	else if( !p[1] && (*p >= '1' && *p <= '4') ) {
 	    unsigned trust;
 	    switch( *p ) {
@@ -143,10 +140,10 @@ _("Could not find a valid trust path to the key.  Let's see whether we\n"
     while( !(rc=enum_trust_web( &context, &lid )) ) {
 	rc = get_ownertrust( lid, &trust );
 	if( rc )
-	    log_fatal("Ooops: couldn't get ownertrust for %lu\n", lid);
+	    log_fatal("Ooops: couldn't get owner trust for %lu\n", lid);
 	if( trust == TRUST_UNDEFINED || trust == TRUST_EXPIRED ||
 	    trust == TRUST_UNKNOWN ) {
-	    if( query_ownertrust( lid ) )
+	    if( edit_ownertrust( lid, 0 ) )
 		any=1;
 	}
     }
@@ -155,7 +152,7 @@ _("Could not find a valid trust path to the key.  Let's see whether we\n"
     enum_trust_web( &context, NULL ); /* close */
 
     if( !any )
-	tty_printf(_("No ownertrust values changed.\n\n") );
+	tty_printf(_("No owner trust values changed.\n\n") );
 
     return rc? rc : any? 0:-1;
 }
@@ -170,18 +167,12 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
     int rc;
 
     if( (trustlevel & TRUST_FLAG_REVOKED) ) {
-	char *answer;
-	int yes;
-
 	log_info("key has been revoked!\n");
 	if( opt.batch )
 	    return 0;
 
-	answer = tty_get("Use this key anyway? ");
-	tty_kill_prompt();
-	yes = answer_is_yes(answer);
-	m_free(answer);
-	if( !yes )
+	if( !cpr_get_answer_is_yes(N_("revoked_key.override"),
+				    _("Use this key anyway? ")) )
 	    return 0;
     }
 
@@ -262,18 +253,14 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
     int rc = do_we_trust( pk, trustlevel );
 
     if( !opt.batch && !rc ) {
-	char *answer;
-
 	tty_printf(_(
 "It is NOT certain that the key belongs to its owner.\n"
 "If you *really* know what you are doing, you may answer\n"
 "the next question with yes\n\n") );
 
-	answer = tty_get("Use this key anyway? ");
-	tty_kill_prompt();
-	if( answer_is_yes(answer) )
+	if( cpr_get_answer_is_yes(N_("untrusted_key.override"),
+				   _("Use this key anyway? "))  )
 	    rc = 1;
-	m_free(answer);
     }
     else if( opt.always_trust && !rc ) {
 	log_info(_("WARNING: Using untrusted key!\n"));
@@ -366,7 +353,7 @@ check_signatures_trust( PKT_signature *sig )
       case TRUST_MARGINAL:
 	write_status( STATUS_TRUST_MARGINAL );
 	log_info(_(
-	 "WARNING: This key is not certified with enough trusted signatures!\n"
+	 "WARNING: This key is not certified with sufficiently trusted signatures!\n"
 		));
 	log_info(_(
 	 "         It is not certain that the signature belongs to the owner.\n"
@@ -418,9 +405,10 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned usage )
 	for(;;) {
 	    rc = 0;
 	    m_free(answer);
-	    answer = tty_get(_("Enter the user ID: "));
+	    answer = cpr_get(N_("pklist.user_id.enter"),
+			      _("Enter the user ID: "));
 	    trim_spaces(answer);
-	    tty_kill_prompt();
+	    cpr_kill_prompt();
 	    if( !*answer )
 		break;
 	    if( pk )
@@ -509,3 +497,102 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned usage )
 }
 
 
+/****************
+ * Return -1 if we could not find an algorithm.
+ */
+int
+select_algo_from_prefs( PK_LIST pk_list, int preftype )
+{
+    PK_LIST pkr;
+    u32 bits[8];
+    byte *pref = NULL;
+    size_t npref;
+    int i, j;
+    int compr_hack=0;
+    int any;
+
+    if( !pk_list )
+	return -1;
+
+    memset( bits, ~0, 8 * sizeof *bits );
+    for( pkr = pk_list; pkr; pkr = pkr->next ) {
+	u32 mask[8];
+
+	memset( mask, 0, 8 * sizeof *mask );
+	if( !pkr->pk->local_id )
+	    BUG(); /* if this occurs, we can use get_ownertrust to set it */
+	if( preftype == PREFTYPE_SYM )
+	    bits[0] = (1<<2); /* 3DES is implicitly there */
+	m_free(pref);
+	pref = get_pref_data( pkr->pk->local_id, pkr->pk->namehash, &npref);
+	any = 0;
+	if( pref ) {
+	    /*log_hexdump("raw: ", pref, npref );*/
+	    for(i=0; i+1 < npref; i+=2 ) {
+		if( pref[i] == preftype ) {
+		    mask[pref[i+1]/32] |= 1 << (pref[i+1]%32);
+		    any = 1;
+		}
+	    }
+	}
+	if( (!pref || !any) && preftype == PREFTYPE_COMPR ) {
+	    mask[0] |= 3; /* asume no_compression and old pgp */
+	    compr_hack = 1;
+	}
+
+	/*log_debug("mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+	       (ulong)mask[7], (ulong)mask[6], (ulong)mask[5], (ulong)mask[4],
+	     (ulong)mask[3], (ulong)mask[2], (ulong)mask[1], (ulong)mask[0]);*/
+	for(i=0; i < 8; i++ )
+	    bits[i] &= mask[i];
+	/*log_debug("bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+	       (ulong)bits[7], (ulong)bits[6], (ulong)bits[5], (ulong)bits[4],
+	     (ulong)bits[3], (ulong)bits[2], (ulong)bits[1], (ulong)bits[0]);*/
+    }
+    /* usable algorithms are now in bits
+     * We now use the last key from pk_list to select
+     * the algorithm we want to use. there are no
+     * preferences for the last key, we select the one
+     * corresponding to first set bit.
+     */
+    i = -1;
+    any = 0;
+    if( pref ) {
+	for(j=0; j+1 < npref; j+=2 ) {
+	    if( pref[j] == preftype ) {
+		any = 1;
+		if( (bits[pref[j+1]/32] & (1<<(pref[j+1]%32))) ) {
+		    i = pref[j+1];
+		    break;
+		}
+	    }
+	}
+    }
+    if( !pref || !any ) {
+	for(j=0; j < 256; j++ )
+	    if( (bits[j/32] & (1<<(j%32))) ) {
+		i = j;
+		break;
+	    }
+    }
+    /*log_debug("prefs of type %d: selected %d\n", preftype, i );*/
+    if( compr_hack && !i ) {
+	/* selected no compression, but we should check whether
+	 * algorithm 1 is also available (the ordering is not relevant
+	 * in this case). */
+	if( bits[0] & (1<<1) )
+	    i = 1;  /* yep; we can use compression algo 1 */
+    }
+
+    if( preftype == PREFTYPE_SYM && i == CIPHER_ALGO_3DES ) {
+	i = CIPHER_ALGO_BLOWFISH;
+	if( opt.verbose )
+	    log_info("replacing 3DES by Blowfish\n");
+    }
+
+
+    m_free(pref);
+    return i;
+}
+
+
