@@ -53,6 +53,8 @@ do_check( PKT_secret_key *sk )
 	    BUG();
 	if( check_cipher_algo( sk->protect.algo ) )
 	    return G10ERR_CIPHER_ALGO; /* unsupported protection algorithm */
+	if( cipher_get_blocksize( sk->protect.algo ) != 8 )
+	    return G10ERR_CIPHER_ALGO; /* unsupported protection algorithm */
 	keyid_from_sk( sk, keyid );
 	dek = passphrase_to_dek( keyid, sk->protect.algo,
 				 &sk->protect.s2k, 0 );
@@ -65,19 +67,54 @@ do_check( PKT_secret_key *sk )
 	memcpy(save_iv, sk->protect.iv, 8 );
 	cipher_decrypt( cipher_hd, sk->protect.iv, sk->protect.iv, 8 );
 	csum = 0;
-	for(i=pubkey_get_npkey(sk->pubkey_algo);
-		i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
-	    buffer = mpi_get_secure_buffer( sk->skey[i], &nbytes, NULL );
-	    cipher_sync( cipher_hd );
-	    assert( mpi_is_protected(sk->skey[i]) );
-	    cipher_decrypt( cipher_hd, buffer, buffer, nbytes );
-	    mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
-	    mpi_clear_protect_flag( sk->skey[i] );
-	    csum += checksum_mpi( sk->skey[i] );
-	    m_free( buffer );
+	if( sk->version >= 4 ) {
+	    int ndata;
+	    byte *p, *data;
+
+
+	    i = pubkey_get_npkey(sk->pubkey_algo);
+	    assert( mpi_is_opaque( sk->skey[i] ) );
+	    p = mpi_get_opaque( sk->skey[i], &ndata );
+	    data = m_alloc_secure( ndata );
+	    cipher_decrypt( cipher_hd, data, p, ndata );
+	    mpi_free( sk->skey[i] ); sk->skey[i] = NULL ;
+	    p = data;
+	    if( ndata < 2 ) {
+		log_error("not enough bytes for checksum\n");
+		sk->csum = 0;
+		csum = 1;
+	    }
+	    else {
+		csum = checksum( data, ndata-2);
+		sk->csum = data[ndata-2] << 8 | data[ndata-1];
+	    }
+	    /* must check it here otherwise the mpi_read_xx would fail
+	     * because the length das an abritary value */
+	    if( sk->csum == csum ) {
+		for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+		    nbytes = ndata;
+		    sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
+		    ndata -= nbytes;
+		    p += nbytes;
+		}
+	    }
+	    m_free(data);
 	}
-	if( opt.emulate_bugs & 1 ) {
-	   csum = sk->csum;
+	else {
+	    for(i=pubkey_get_npkey(sk->pubkey_algo);
+		    i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+		buffer = mpi_get_secure_buffer( sk->skey[i], &nbytes, NULL );
+		cipher_sync( cipher_hd );
+		assert( mpi_is_protected(sk->skey[i]) );
+		cipher_decrypt( cipher_hd, buffer, buffer, nbytes );
+		mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
+		mpi_clear_protect_flag( sk->skey[i] );
+		csum += checksum_mpi( sk->skey[i] );
+		m_free( buffer );
+	    }
+	    if( opt.emulate_bugs & EMUBUG_GPGCHKSUM ) {
+	       csum = sk->csum;
+	    }
 	}
 	cipher_close( cipher_hd );
 	/* now let's see whether we have used the right passphrase */
@@ -167,7 +204,7 @@ is_secret_key_protected( PKT_secret_key *sk )
 int
 protect_secret_key( PKT_secret_key *sk, DEK *dek )
 {
-    int i, rc = 0;
+    int i,j, rc = 0;
     byte *buffer;
     unsigned nbytes;
     u16 csum;
@@ -180,27 +217,75 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 
 	if( check_cipher_algo( sk->protect.algo ) )
 	    rc = G10ERR_CIPHER_ALGO; /* unsupport protection algorithm */
+	else if( cipher_get_blocksize( sk->protect.algo ) != 8 )
+	    rc = G10ERR_CIPHER_ALGO; /* unsupport protection algorithm */
 	else {
 	    cipher_hd = cipher_open( sk->protect.algo,
 				     CIPHER_MODE_AUTO_CFB, 1 );
 	    cipher_setkey( cipher_hd, dek->key, dek->keylen );
 	    cipher_setiv( cipher_hd, NULL );
 	    cipher_encrypt( cipher_hd, sk->protect.iv, sk->protect.iv, 8 );
-	    /* NOTE: we always recalculate the checksum because there are some
-	     * test releases which calculated it wrong */
-	    csum = 0;
-	    for(i=pubkey_get_npkey(sk->pubkey_algo);
-		    i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
-		csum += checksum_mpi_counted_nbits( sk->skey[i] );
-		buffer = mpi_get_buffer( sk->skey[i], &nbytes, NULL );
-		cipher_sync( cipher_hd );
-		assert( !mpi_is_protected(sk->skey[i]) );
-		cipher_encrypt( cipher_hd, buffer, buffer, nbytes );
-		mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
-		mpi_set_protect_flag( sk->skey[i] );
-		m_free( buffer );
+	    if( sk->version >= 4 ) {
+	      #define NMPIS (PUBKEY_MAX_NSKEY - PUBKEY_MAX_NPKEY)
+		byte *buffer[NMPIS];
+		unsigned nbytes[NMPIS];
+		unsigned nbits[NMPIS];
+		int ndata=0;
+		byte *p, *data;
+
+		for(j=0, i = pubkey_get_npkey(sk->pubkey_algo);
+			i < pubkey_get_nskey(sk->pubkey_algo); i++, j++ ) {
+		    assert( !mpi_is_opaque( sk->skey[i] ) );
+		    buffer[j] = mpi_get_buffer( sk->skey[i], &nbytes[j], NULL );
+		    nbits[j]  = mpi_get_nbits( sk->skey[i] );
+		    ndata += nbytes[j] + 2;
+		}
+		for( ; j < NMPIS; j++ )
+		    buffer[j] = NULL;
+		ndata += 2; /* for checksum */
+
+		data = m_alloc_secure( ndata );
+		p = data;
+		for(j=0; j < NMPIS && buffer[j]; j++ ) {
+		    p[0] = nbits[j] >> 8 ;
+		    p[1] = nbits[j];
+		    p += 2;
+		    memcpy(p, buffer[j], nbytes[j] );
+		    p += nbytes[j];
+		    m_free(buffer[j]);
+		}
+	      #undef NMPIS
+		csum = checksum( data, ndata-2);
+		sk->csum = csum;
+		*p++ =	csum >> 8;
+		*p++ =	csum;
+		assert( p == data+ndata );
+		cipher_encrypt( cipher_hd, data, data, ndata );
+		for(i = pubkey_get_npkey(sk->pubkey_algo);
+			i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+		    mpi_free( sk->skey[i] );
+		    sk->skey[i] = NULL;
+		}
+		i = pubkey_get_npkey(sk->pubkey_algo);
+		sk->skey[i] = mpi_set_opaque(NULL, data, ndata );
+	    }
+	    else {
+		/* NOTE: we always recalculate the checksum because there
+		 * are some test releases which calculated it wrong */
+		csum = 0;
+		for(i=pubkey_get_npkey(sk->pubkey_algo);
+			i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
+		    csum += checksum_mpi_counted_nbits( sk->skey[i] );
+		    buffer = mpi_get_buffer( sk->skey[i], &nbytes, NULL );
+		    cipher_sync( cipher_hd );
+		    assert( !mpi_is_protected(sk->skey[i]) );
+		    cipher_encrypt( cipher_hd, buffer, buffer, nbytes );
+		    mpi_set_buffer( sk->skey[i], buffer, nbytes, 0 );
+		    mpi_set_protect_flag( sk->skey[i] );
+		    m_free( buffer );
+		}
+		sk->csum = csum;
 	    }
-	    sk->csum = csum;
 	    sk->is_protected = 1;
 	    cipher_close( cipher_hd );
 	}
