@@ -51,7 +51,7 @@ encode_session_key( DEK *dek, unsigned nbits )
     int nframe = (nbits+7) / 8;
     byte *p;
     byte *frame;
-    int i,n,c;
+    int i,n;
     u16 csum;
     MPI a;
 
@@ -86,11 +86,28 @@ encode_session_key( DEK *dek, unsigned nbits )
     frame[n++] = 2;
     i = nframe - 6 - dek->keylen;
     assert( i > 0 );
-    for( ; i ; i-- ) {
-	while( !(c = get_random_byte(1)) )
-	    ;
-	frame[n++] = c;
+    p = get_random_bits( i*8, 1, 1 );
+    /* replace zero bytes by new values */
+    for(;;) {
+	int j, k;
+	byte *pp;
+
+	/* count the zero bytes */
+	for(j=k=0; j < i; j++ )
+	    if( !p[j] )
+		k++;
+	if( !k )
+	    break; /* okay: no zero bytes */
+	k += k/128; /* better get some more */
+	pp = get_random_bits( k*8, 1, 1);
+	for(j=0; j < i && k ; j++ )
+	    if( !p[j] )
+		p[j] = pp[--k];
+	m_free(pp);
     }
+    memcpy( frame+n, p, i );
+    m_free(p);
+    n += i;
     frame[n++] = 0;
     frame[n++] = dek->algo;
     memcpy( frame+n, dek->key, dek->keylen ); n += dek->keylen;
