@@ -35,6 +35,7 @@
 #include "main.h"
 #include "filter.h"
 #include "ttyio.h"
+#include "trustdb.h"
 #include "i18n.h"
 
 
@@ -118,6 +119,8 @@ only_old_style( SK_LIST sk_list )
     return old_style;
 }
 
+
+
 /****************
  * Sign the files whose names are in FILENAME.
  * If DETACHED has the value true,
@@ -150,6 +153,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     SK_LIST sk_rover = NULL;
     int multifile = 0;
     int old_style = opt.rfc1991;
+    int compr_algo = -1; /* unknown */
 
 
     memset( &afx, 0, sizeof afx);
@@ -176,6 +180,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( encrypt ) {
 	if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC )) )
 	    goto leave;
+	if( !old_style )
+	    compr_algo = select_algo_from_prefs( pk_list, PREFTYPE_COMPR );
     }
 
     /* prepare iobufs */
@@ -226,17 +232,32 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_push_filter( out, encrypt_filter, &efx );
     }
 
-    if( opt.compress && !outfile ) {
-	if( old_style )
-	    zfx.algo = 1;
-	iobuf_push_filter( out, compress_filter, &zfx );
+    if( opt.compress && !outfile && ( !detached || opt.compress_sigs) ) {
+	if( !compr_algo )
+	    ; /* don't use compression */
+	else {
+	    if( old_style || compr_algo == 1 )
+		zfx.algo = 1;
+	    iobuf_push_filter( out, compress_filter, &zfx );
+	}
     }
 
     if( !detached && !old_style ) {
-	/* loop over the secret certificates and build headers */
-	for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	int skcount=0;
+	/* loop over the secret certificates and build headers
+	 * The specs now say that the data should be bracket by
+	 * the onepass-sig and signature-packet; so we muts build it
+	 * here in reverse order */
+	for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next )
+	    skcount++;
+	for( ; skcount; skcount-- ) {
 	    PKT_secret_key *sk;
 	    PKT_onepass_sig *ops;
+	    int i = 0;
+
+	    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next )
+		if( ++i == skcount )
+		    break;
 
 	    sk = sk_rover->sk;
 	    ops = m_alloc_clear( sizeof *ops );
@@ -244,7 +265,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	    ops->digest_algo = hash_for(sk->pubkey_algo);
 	    ops->pubkey_algo = sk->pubkey_algo;
 	    keyid_from_sk( sk, ops->keyid );
-	    ops->last = !sk_rover->next;
+	    ops->last = skcount == 1;
 
 	    init_packet(&pkt);
 	    pkt.pkttype = PKT_ONEPASS_SIG;
@@ -259,7 +280,6 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	}
     }
 
-
     /* setup the inner packet */
     if( detached ) {
 	if( multifile ) {
@@ -650,4 +670,112 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 }
 
 
+/****************
+ * Create a signature packet for the given public key certificate
+ * and the user id and return it in ret_sig. User signature class SIGCLASS
+ * user-id is not used (and may be NULL if sigclass is 0x20)
+ * If digest_algo is 0 the function selects an appropriate one.
+ */
+int
+make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
+		    PKT_user_id *uid, PKT_public_key *subpk,
+		    PKT_secret_key *sk,
+		    int sigclass, int digest_algo,
+		    int (*mksubpkt)(PKT_signature *, void *), void *opaque
+		   )
+{
+    PKT_signature *sig;
+    int rc=0;
+    MD_HANDLE md;
+
+    assert( (sigclass >= 0x10 && sigclass <= 0x13)
+	    || sigclass == 0x20 || sigclass == 0x18 );
+    if( !digest_algo ) {
+	switch( sk->pubkey_algo ) {
+	  case PUBKEY_ALGO_DSA: digest_algo = DIGEST_ALGO_SHA1; break;
+	  case PUBKEY_ALGO_RSA_S:
+	  case PUBKEY_ALGO_RSA: digest_algo = DIGEST_ALGO_MD5; break;
+	  default:		digest_algo = DIGEST_ALGO_RMD160; break;
+	}
+    }
+    md = md_open( digest_algo, 0 );
+
+    /* hash the public key certificate and the user id */
+    hash_public_key( md, pk );
+    if( sigclass == 0x18 ) { /* subkey binding */
+	hash_public_key( md, subpk );
+    }
+    else if( sigclass != 0x20 ) {
+	if( sk->version >=4 ) {
+	    byte buf[5];
+	    buf[0] = 0xb4;	      /* indicates a userid packet */
+	    buf[1] = uid->len >> 24;  /* always use 4 length bytes */
+	    buf[2] = uid->len >> 16;
+	    buf[3] = uid->len >>  8;
+	    buf[4] = uid->len;
+	    md_write( md, buf, 5 );
+	}
+	md_write( md, uid->name, uid->len );
+    }
+    /* and make the signature packet */
+    sig = m_alloc_clear( sizeof *sig );
+    sig->version = sk->version;
+    keyid_from_sk( sk, sig->keyid );
+    sig->pubkey_algo = sk->pubkey_algo;
+    sig->digest_algo = digest_algo;
+    sig->timestamp = make_timestamp();
+    sig->sig_class = sigclass;
+    if( sig->version >= 4 )
+	build_sig_subpkt_from_sig( sig );
+
+    if( sig->version >= 4 && mksubpkt )
+	rc = (*mksubpkt)( sig, opaque );
+
+    if( !rc ) {
+	if( sig->version >= 4 )
+	    md_putc( md, sig->version );
+	md_putc( md, sig->sig_class );
+	if( sig->version < 4 ) {
+	    u32 a = sig->timestamp;
+	    md_putc( md, (a >> 24) & 0xff );
+	    md_putc( md, (a >> 16) & 0xff );
+	    md_putc( md, (a >>	8) & 0xff );
+	    md_putc( md,  a	   & 0xff );
+	}
+	else {
+	    byte buf[6];
+	    size_t n;
+
+	    md_putc( md, sig->pubkey_algo );
+	    md_putc( md, sig->digest_algo );
+	    if( sig->hashed_data ) {
+		n = (sig->hashed_data[0] << 8) | sig->hashed_data[1];
+		md_write( md, sig->hashed_data, n+2 );
+		n += 6;
+	    }
+	    else
+		n = 6;
+	    /* add some magic */
+	    buf[0] = sig->version;
+	    buf[1] = 0xff;
+	    buf[2] = n >> 24; /* hmmm, n is only 16 bit, so this is always 0 */
+	    buf[3] = n >> 16;
+	    buf[4] = n >>  8;
+	    buf[5] = n;
+	    md_write( md, buf, 6 );
+
+	}
+	md_final(md);
+
+	rc = complete_sig( sig, sk, md );
+    }
+
+    md_close( md );
+    if( rc )
+	free_seckey_enc( sig );
+    else
+	*ret_sig = sig;
+    return rc;
+}
+
 
